"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
class Myo {
    constructor(macAddress, name, connectIndex, myoManager) {
        this.macAddress = macAddress;
        this.name = name;
        this.connectIndex = connectIndex;
        this.myoManager = myoManager;
        this.isLocked = true;
        this.isConnected = false;
        this.isSynced = false;
        this.batteryLevel = 0;
        this.orientationOffset = util_1.Quaternion.Identity();
        this.lastQuant = util_1.Quaternion.Identity();
        this.lastPose = undefined;
    }
    lock() {
        this.myoManager.sendCommand({
            command: "lock",
            myo: this.connectIndex
        });
    }
    unlock(hold = false) {
        this.myoManager.sendCommand({
            command: "unlock",
            myo: this.connectIndex,
            type: hold ? "hold" : "timed"
        });
    }
    zeroOrientation() {
        this.orientationOffset = this.lastQuant.invert();
        this.myoManager.emit("zero_orientation" /* ZeroOrientation */, this);
    }
    vibrate(duration = "medium" /* Medium */) {
        this.myoManager.sendCommand({
            command: "vibrate",
            myo: this.connectIndex,
            type: duration
        });
    }
    requestBluetoothStrength() {
        this.myoManager.sendCommand({
            command: "request_rssi",
            myo: this.connectIndex
        });
    }
    requestBatteryLevel() {
        this.myoManager.sendCommand({
            command: "request_battery_level",
            myo: this.connectIndex
        });
    }
    streamEMG(enabled) {
        this.myoManager.sendCommand({
            command: "set_stream_emg",
            myo: this.connectIndex,
            type: enabled ? "enabled" : "disabled"
        });
    }
    pose(data) {
        const { pose } = data;
        if (this.lastPose) {
            // @ts-ignore: Runtime string type checking currently unsupported in TS
            this.myoManager.emit(`${this.lastPose}_off`);
            this.myoManager.emit("pose_off" /* PoseLeave */, this, this.lastPose);
        }
        if (pose === "rest" /* Rest */) {
            this.myoManager.emit("rest" /* Rest */, this);
            this.lastPose = undefined;
            if (this.myoManager.lockingPolicy === "standard" /* Standard */)
                this.unlock();
        }
        else {
            this.myoManager.emit(pose, this);
            this.myoManager.emit("pose" /* PoseEnter */, this, pose);
            this.lastPose = pose;
            if (this.myoManager.lockingPolicy === "standard" /* Standard */)
                this.unlock(true);
        }
    }
    orientation(data) {
        this.lastQuant = data.orientation;
        const orientation = this.orientationOffset.rotate(data.orientation);
        const accelerometer = util_1.Vector3.fromArray(data.accelerometer);
        const gyroscope = util_1.Vector3.fromArray(data.gyroscope);
        const imuData = { orientation, accelerometer, gyroscope };
        const { timestamp } = data;
        this.myoManager.emit("orientation" /* Orientation */, this, orientation, timestamp);
        this.myoManager.emit("accelerometer" /* Accelerometer */, this, accelerometer, timestamp);
        this.myoManager.emit("gyroscope" /* Gyroscope */, this, gyroscope, timestamp);
        this.myoManager.emit("imu" /* IMU */, this, imuData, timestamp);
        this.lastIMU = imuData;
    }
    //Status Events
    syncArm(data) {
        this.arm = data.arm;
        this.direction = data.x_direction;
        this.warmupState = data.warmup_state;
        this.isSynced = true;
    }
    unsyncArm() {
        this.arm = undefined;
        this.direction = undefined;
        this.warmupState = undefined;
        this.isSynced = false;
    }
    updateBluetooth(data) {
        const { timestamp } = data;
        const strength = util_1.getStrengthFromRssi(data.rssi);
        this.myoManager.emit("bluetooth_strength" /* BluetoothStrength */, this, strength, timestamp);
        this.myoManager.emit("rssi" /* RSSI */, this, data.rssi, timestamp);
        this.myoManager.emit("status" /* Status */, this, data, timestamp);
    }
    updateBatteryLevel(data) {
        this.batteryLevel = data.battery_level;
        this.myoManager.emit("battery_level" /* BatteryLevel */, this, data.battery_level, data.timestamp);
        this.myoManager.emit("status" /* Status */, this, data, data.timestamp);
    }
}
exports.Myo = Myo;
//# sourceMappingURL=Myo.js.map